import queue
from rupython import __Общие_функции__

Очередь_пуста = queue.Empty
Очередь_полна = queue.Full

Словарь_параметров = {
    'максимальный_размер': 'maxsize',
    'элемент': 'item',
    'блоковать': 'block',
    'таймаут': 'timeout',
    'приоритет': 'priority'
}

Словарь_склонений = {
    'элемент': ['элемент', 'элемента', 'элементов'],
    'задача': ['задача', 'задачи', 'задач']
}

class Очередь:
    def __init__(здесь, максимальный_размер = 0):
        kwargs = {'maxsize': максимальный_размер}
        здесь._queue = queue.Queue(**kwargs)

    def Добавить(здесь, элемент, блоковать = True, таймаут = None):
        kwargs = {'item': элемент, 'block': блоковать}
        if таймаут is not None:
            kwargs['timeout'] = таймаут
        здесь._queue.put(**kwargs)

    def Добавить_без_блокования(здесь, элемент):
        здесь._queue.put_nowait(элемент)

    def Получить(здесь, блоковать = True, таймаут = None):
        kwargs = {'block': блоковать}
        if таймаут is not None:
            kwargs['timeout'] = таймаут
        return здесь._queue.get(**kwargs)

    def Получить_без_блокования(здесь):
        return здесь._queue.get_nowait()

    def Задача_выполнена(здесь):
        здесь._queue.task_done()

    def Ждать_завершения(здесь):
        здесь._queue.join()

    def Пустая(здесь):
        return здесь._queue.empty()

    def Полная(здесь):
        return здесь._queue.full()

    def Размер(здесь):
        return здесь._queue.qsize()

    def __len__(здесь):
        return здесь._queue.qsize()

    def __str__(здесь):
        размер = здесь.Размер()
        return f"Очередь с {__Общие_функции__.Склонение_после_числительного(размер, Словарь_склонений['элемент'])}"

    def В_строку_с_задачами(здесь):
        размер = здесь.Размер()
        return f"Очередь с {__Общие_функции__.Склонение_после_числительного(размер, Словарь_склонений['задача'])}"

class Стековая_очередь:
    def __init__(здесь, максимальный_размер = 0):
        kwargs = {'maxsize': максимальный_размер}
        здесь._queue = queue.LifoQueue(**kwargs)

    def Добавить(здесь, элемент, блоковать = True, таймаут = None):
        kwargs = {'item': элемент, 'block': блоковать}
        if таймаут is not None:
            kwargs['timeout'] = таймаут
        здесь._queue.put(**kwargs)

    def Добавить_без_блокования(здесь, элемент):
        здесь._queue.put_nowait(элемент)

    def Получить(здесь, блоковать = True, таймаут = None):
        kwargs = {'block': блоковать}
        if таймаут is not None:
            kwargs['timeout'] = таймаут
        return здесь._queue.get(**kwargs)

    def Получить_без_блокования(здесь):
        return здесь._queue.get_nowait()

    def Задача_выполнена(здесь):
        здесь._queue.task_done()

    def Ждать_завершения(здесь):
        здесь._queue.join()

    def Пустая(здесь):
        return здесь._queue.empty()

    def Полная(здесь):
        return здесь._queue.full()

    def Размер(здесь):
        return здесь._queue.qsize()

    def __len__(здесь):
        return здесь._queue.qsize()

    def __str__(здесь):
        размер = здесь.Размер()
        return f"Стековая_очередь с {__Общие_функции__.Склонение_после_числительного(размер, Словарь_склонений['элемент'])}"

    def В_строку_с_задачами(здесь):
        размер = здесь.Размер()
        return f"Стековая_очередь с {__Общие_функции__.Склонение_после_числительного(размер, Словарь_склонений['задача'])}"

class Приоритетная_очередь:
    def __init__(здесь, максимальный_размер = 0):
        kwargs = {'maxsize': максимальный_размер}
        здесь._queue = queue.PriorityQueue(**kwargs)

    def Добавить(здесь, элемент, приоритет, блоковать = True, таймаут = None):
        kwargs = {'item': (приоритет, элемент), 'block': блоковать}
        if таймаут is not None:
            kwargs['timeout'] = таймаут
        здесь._queue.put(**kwargs)

    def Добавить_без_блокования(здесь, элемент, приоритет):
        здесь._queue.put_nowait((приоритет, элемент))

    def Получить(здесь, блоковать = True, таймаут = None):
        kwargs = {'block': блоковать}
        if таймаут is not None:
            kwargs['timeout'] = таймаут
        приоритет, элемент = здесь._queue.get(**kwargs)
        return элемент

    def Получить_с_приоритетом(здесь, блоковать = True, таймаут = None):
        kwargs = {'block': блоковать}
        if таймаут is not None:
            kwargs['timeout'] = таймаут
        return здесь._queue.get(**kwargs)

    def Получить_без_блокования(здесь):
        приоритет, элемент = здесь._queue.get_nowait()
        return элемент

    def Получить_без_блокования_с_приоритетом(здесь):
        """Возвращает кортеж (приоритет, элемент)."""
        return здесь._queue.get_nowait()

    def Задача_выполнена(здесь):
        здесь._queue.task_done()

    def Ждать_завершения(здесь):
        здесь._queue.join()

    def Пустая(здесь):
        return здесь._queue.empty()

    def Полная(здесь):
        return здесь._queue.full()

    def Размер(здесь):
        return здесь._queue.qsize()

    def __len__(здесь):
        return здесь._queue.qsize()

    def __str__(здесь):
        размер = здесь.Размер()
        return f"Приоритетная_очередь с {__Общие_функции__.Склонение_после_числительного(размер, Словарь_склонений['элемент'])}"

    def В_строку_с_задачами(здесь):
        размер = здесь.Размер()
        return f"Приоритетная_очередь с {__Общие_функции__.Склонение_после_числительного(размер, Словарь_склонений['задача'])}"
